window.Gleam.initSearch([{"doc":"argamak","title":"argamak","content":"# argamak 🐎\n\n[![Hex Package](https://img.shields.io/hexpm/v/argamak?color=ffaff3&amp;label=%F0%9F%93%A6)](https://hex.pm/packages/argamak)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3?label=%F0%9F%93%9A)](https://hexdocs.pm/argamak/)\n[![License](https://img.shields.io/hexpm/l/argamak?color=ffaff3&amp;label=%F0%9F%93%83)](https://github.com/tynanbe/argamak/blob/main/LICENSE)\n[![Build](https://img.shields.io/github/workflow/status/tynanbe/argamak/CI?color=ffaff3&amp;label=%E2%9C%A8)](https://github.com/tynanbe/argamak/actions)\n\nA Gleam library for tensor maths.\n\n&gt; “I admire the elegance of your method of computation; it must be nice to ride\n&gt; through these fields upon the horse of true mathematics while the like of us\n&gt; have to make our way laboriously on foot.”\n&gt;\n&gt; —Albert Einstein, to Tullio Levi-Civita, circa 1915–1917\n\n&lt;p align=&quot;center&quot; width=&quot;100%&quot;&gt;&lt;a href=&quot;https://www.wikiwand.com/en/Akhal-Teke&quot;&gt;&lt;img alt=&quot;Argamak: A shiny steed.&quot; src=&quot;https://github.com/tynanbe/argamak/raw/main/argamak.jpg&quot; width=&quot;250&quot;&gt;&lt;/a&gt;&lt;/p&gt;\n\n## Installation\n\n### As a dependency of your Gleam project\n\n• Add `argamak` to `gleam.toml` from the command line\n\n```shell\n$ gleam add argamak\n```\n\n### As a dependency of your Mix project\n\n• Add `argamak` to `mix.exs`\n\n```elixir\ndefp deps do\n  [\n    {:argamak, &quot;~&gt; 0.2&quot;},\n  ]\nend\n```\n\n### As a dependency of your Rebar3 project\n\n• Add `argamak` to `rebar.config`\n\n```erlang\n{deps, [\n  {argamak, &quot;0.2.0&quot;}\n]}.\n```\n\n### JavaScript\n\nThe `@tensorflow/tfjs` package is a runtime requirement for `argamak`, and its\nimport path in the `argamak_ffi.mjs` module might need adjustment, depending on\nyour use case.\n\n## Usage\n\n```gleam\n// derby.gleam\nimport argamak/space\nimport argamak/tensor\n\npub type Axis {\n  Horse\n  Trial\n}\n\npub fn print_times(from list: List(Float)) {\n  assert Ok(space) = space.d2(#(Horse, 3), #(Trial, -1))\n  try tensor = tensor.from_floats(of: list, into: space)\n\n  tensor\n  |&gt; tensor.print(wrap_at: -1, meta: True)\n  |&gt; Ok\n}\n```\n\n### Example\n\n```gleam\n&gt; derby.print_times(from: [1.2, 1.3, 1.3, 1., 1.5, 0.9])\n// Tensor\n// format: Float32\n// space: D2 #(Horse, 3), #(Trial, 2)\n// data:\n// [[1.2, 1.3],\n//  [1.3, 1.0],\n//  [1.5, 0.9]]\nOk(Nil)\n```\n","url":"index.html"},{"doc":"argamak/util","title":"UtilError","content":"pub type UtilError {\n  InvalidRecord\n}\n An error type for utility functions.\nInvalidRecord\n\n\nSynonyms:\nargamak/util.UtilError\nargamak/util UtilError","url":"argamak/util.html#UtilError"},{"doc":"argamak/util","title":"record_to_string","content":"pub fn record_to_string(record: a) -&gt; Result(String, UtilError)\n Results in a `String` converted from a custom type record&#39;s name on success,\n or a `UtilError` on failure.\n\n ## Examples\n\n \n &gt; type Axis { X }\n &gt; record_to_string(X)\n Ok(&quot;X&quot;)\n\n &gt; record_to_string(3)\n Error(InvalidRecord)\n \n\nSynonyms:\nargamak/util.record_to_string\nargamak/util record_to_string","url":"argamak/util.html#record_to_string"},{"doc":"argamak/util","title":"argamak/util","content":"","url":"argamak/util.html"},{"doc":"argamak/format","title":"Format","content":"pub opaque type Format(a)\n Numerical formats for tensors.\n\n Each `Format` uses a specific number of bits to represent a data point.\n\n\nSynonyms:\nargamak/format.Format\nargamak/format Format","url":"argamak/format.html#Format"},{"doc":"argamak/format","title":"Native","content":"pub opaque type Native\n A type for `Native` format representations.\n\n\nSynonyms:\nargamak/format.Native\nargamak/format Native","url":"argamak/format.html#Native"},{"doc":"argamak/format","title":"bfloat16","content":"pub fn bfloat16() -&gt; Format(Float)\n Creates a 16-bit brain floating-point format.\n\nSynonyms:\nargamak/format.bfloat16\nargamak/format bfloat16","url":"argamak/format.html#bfloat16"},{"doc":"argamak/format","title":"float16","content":"pub fn float16() -&gt; Format(Float)\n Creates a 16-bit floating-point `Format`.\n\nSynonyms:\nargamak/format.float16\nargamak/format float16","url":"argamak/format.html#float16"},{"doc":"argamak/format","title":"float32","content":"pub fn float32() -&gt; Format(Float)\n Creates a 32-bit floating-point `Format`.\n\nSynonyms:\nargamak/format.float32\nargamak/format float32","url":"argamak/format.html#float32"},{"doc":"argamak/format","title":"float64","content":"pub fn float64() -&gt; Format(Float)\n Creates a 64-bit floating-point `Format`.\n\nSynonyms:\nargamak/format.float64\nargamak/format float64","url":"argamak/format.html#float64"},{"doc":"argamak/format","title":"int16","content":"pub fn int16() -&gt; Format(Int)\n Creates a 16-bit signed integer `Format`.\n\nSynonyms:\nargamak/format.int16\nargamak/format int16","url":"argamak/format.html#int16"},{"doc":"argamak/format","title":"int32","content":"pub fn int32() -&gt; Format(Int)\n Creates a 32-bit signed integer `Format`.\n\nSynonyms:\nargamak/format.int32\nargamak/format int32","url":"argamak/format.html#int32"},{"doc":"argamak/format","title":"int64","content":"pub fn int64() -&gt; Format(Int)\n Creates a 64-bit signed integer `Format`.\n\nSynonyms:\nargamak/format.int64\nargamak/format int64","url":"argamak/format.html#int64"},{"doc":"argamak/format","title":"int8","content":"pub fn int8() -&gt; Format(Int)\n Creates an 8-bit signed integer `Format`.\n\nSynonyms:\nargamak/format.int8\nargamak/format int8","url":"argamak/format.html#int8"},{"doc":"argamak/format","title":"to_native","content":"pub fn to_native(format: Format(a)) -&gt; Native\n Converts a given `Format` into its native representation.\n\nSynonyms:\nargamak/format.to_native\nargamak/format to_native","url":"argamak/format.html#to_native"},{"doc":"argamak/format","title":"to_string","content":"pub fn to_string(format: Format(a)) -&gt; String\n A type for `Native` format representations.\n\n Converts a `Format` into a `String`.\n\nSynonyms:\nargamak/format.to_string\nargamak/format to_string","url":"argamak/format.html#to_string"},{"doc":"argamak/format","title":"uint16","content":"pub fn uint16() -&gt; Format(Int)\n Creates a 16-bit unsigned integer `Format`.\n\nSynonyms:\nargamak/format.uint16\nargamak/format uint16","url":"argamak/format.html#uint16"},{"doc":"argamak/format","title":"uint32","content":"pub fn uint32() -&gt; Format(Int)\n Creates a 32-bit unsigned integer `Format`.\n\nSynonyms:\nargamak/format.uint32\nargamak/format uint32","url":"argamak/format.html#uint32"},{"doc":"argamak/format","title":"uint64","content":"pub fn uint64() -&gt; Format(Int)\n Creates a 64-bit unsigned integer `Format`.\n\nSynonyms:\nargamak/format.uint64\nargamak/format uint64","url":"argamak/format.html#uint64"},{"doc":"argamak/format","title":"uint8","content":"pub fn uint8() -&gt; Format(Int)\n Creates an 8-bit unsigned integer `Format`.\n\nSynonyms:\nargamak/format.uint8\nargamak/format uint8","url":"argamak/format.html#uint8"},{"doc":"argamak/format","title":"argamak/format","content":"","url":"argamak/format.html"},{"doc":"argamak/space","title":"D0","content":"pub type D0 {\n  D0\n}\n A type without dimensions.\n\nD0\n\n\nSynonyms:\nargamak/space.D0\nargamak/space D0","url":"argamak/space.html#D0"},{"doc":"argamak/space","title":"D1","content":"pub type D1 {\n  D1\n}\n A type with one dimension.\n\nD1\n\n\nSynonyms:\nargamak/space.D1\nargamak/space D1","url":"argamak/space.html#D1"},{"doc":"argamak/space","title":"D2","content":"pub type D2 {\n  D2\n}\n A type with two dimensions.\n\nD2\n\n\nSynonyms:\nargamak/space.D2\nargamak/space D2","url":"argamak/space.html#D2"},{"doc":"argamak/space","title":"D3","content":"pub type D3 {\n  D3\n}\n A type with three dimensions.\n\nD3\n\n\nSynonyms:\nargamak/space.D3\nargamak/space D3","url":"argamak/space.html#D3"},{"doc":"argamak/space","title":"D4","content":"pub type D4 {\n  D4\n}\n A type with four dimensions.\n\nD4\n\n\nSynonyms:\nargamak/space.D4\nargamak/space D4","url":"argamak/space.html#D4"},{"doc":"argamak/space","title":"D5","content":"pub type D5 {\n  D5\n}\n A type with five dimensions.\n\nD5\n\n\nSynonyms:\nargamak/space.D5\nargamak/space D5","url":"argamak/space.html#D5"},{"doc":"argamak/space","title":"D6","content":"pub type D6 {\n  D6\n}\n A type with six dimensions.\n\nD6\n\n\nSynonyms:\nargamak/space.D6\nargamak/space D6","url":"argamak/space.html#D6"},{"doc":"argamak/space","title":"Space","content":"pub opaque type Space(dn, axis)\n An n-dimensional `Space` with axis–shape tuple elements, one per dimension.\n\n\nSynonyms:\nargamak/space.Space\nargamak/space Space","url":"argamak/space.html#Space"},{"doc":"argamak/space","title":"SpaceError","content":"pub type SpaceError {\n  SpaceError(error: String, element: String)\n}\n An error returned when attempting to create an invalid `Space`.\n\nSpaceError(error: String, element: String)\n\n\nSynonyms:\nargamak/space.SpaceError\nargamak/space SpaceError","url":"argamak/space.html#SpaceError"},{"doc":"argamak/space","title":"SpaceErrors","content":"pub type SpaceErrors =\n  List(SpaceError)\n A `SpaceError` list.\n\n\nSynonyms:\nargamak/space.SpaceErrors\nargamak/space SpaceErrors","url":"argamak/space.html#SpaceErrors"},{"doc":"argamak/space","title":"axes","content":"pub fn axes(of space: Space(a, b)) -&gt; List(b)\n Returns the axes of a given `Space`.\n\n ## Examples\n\n \n &gt; assert Ok(space) = d0()\n &gt; axes(space)\n []\n\n &gt; type Axis { A B C }\n &gt; assert Ok(space) = d1(#(A, -1))\n &gt; axes(space)\n [A]\n\n &gt; assert Ok(space) = d3(#(A, 2), #(B, 2), #(C, -1))\n &gt; axes(space)\n [A, B, C]\n \n\nSynonyms:\nargamak/space.axes\nargamak/space axes","url":"argamak/space.html#axes"},{"doc":"argamak/space","title":"d0","content":"pub fn d0() -&gt; Result(Space(D0, a), List(SpaceError))\n Results in a dimensionless `Space`.\n\n ## Examples\n\n \n &gt; assert Ok(space) = d0()\n &gt; elements(space)\n []\n \n\nSynonyms:\nargamak/space.d0\nargamak/space d0","url":"argamak/space.html#d0"},{"doc":"argamak/space","title":"d1","content":"pub fn d1(a: #(a, Int)) -&gt; Result(Space(D1, a), List(SpaceError))\n Results in a 1-dimensional `Space` on success, or `SpaceErrors` on failure.\n\n ## Examples\n\n \n &gt; type Axis { A }\n &gt; assert Ok(space) = d1(#(A, -1))\n &gt; elements(space)\n [#(A, -1)]\n \n\nSynonyms:\nargamak/space.d1\nargamak/space d1","url":"argamak/space.html#d1"},{"doc":"argamak/space","title":"d2","content":"pub fn d2(a: #(a, Int), b: #(a, Int)) -&gt; Result(\n  Space(D2, a),\n  List(SpaceError),\n)\n Results in a 2-dimensional `Space` on success, or `SpaceErrors` on failure.\n\n ## Examples\n\n \n &gt; type Axis { A B }\n &gt; assert Ok(space) = d2(#(A, 2), #(B, 2))\n &gt; elements(space)\n [#(A, 2), #(B, 2)]\n \n\nSynonyms:\nargamak/space.d2\nargamak/space d2","url":"argamak/space.html#d2"},{"doc":"argamak/space","title":"d3","content":"pub fn d3(a: #(a, Int), b: #(a, Int), c: #(a, Int)) -&gt; Result(\n  Space(D3, a),\n  List(SpaceError),\n)\n Results in a 3-dimensional `Space` on success, or `SpaceErrors` on failure.\n\n ## Examples\n\n \n &gt; type Axis { A B C }\n &gt; assert Ok(space) = d3(#(A, 2), #(B, 2), #(C, -1))\n &gt; elements(space)\n [#(A, 2), #(B, 2), #(C, -1)]\n \n\nSynonyms:\nargamak/space.d3\nargamak/space d3","url":"argamak/space.html#d3"},{"doc":"argamak/space","title":"d4","content":"pub fn d4(a: #(a, Int), b: #(a, Int), c: #(a, Int), d: #(a, Int)) -&gt; Result(\n  Space(D4, a),\n  List(SpaceError),\n)\n Results in a 4-dimensional `Space` on success, or `SpaceErrors` on failure.\n\n ## Examples\n\n \n &gt; type Axis { A B C D }\n &gt; assert Ok(space) = d4(#(A, 2), #(B, 2), #(C, -1), #(D, 1))\n &gt; elements(space)\n [#(A, 2), #(B, 2), #(C, -1), #(D, 1)]\n \n\nSynonyms:\nargamak/space.d4\nargamak/space d4","url":"argamak/space.html#d4"},{"doc":"argamak/space","title":"d5","content":"pub fn d5(a: #(a, Int), b: #(a, Int), c: #(a, Int), d: #(a, Int), e: #(\n    a,\n    Int,\n  )) -&gt; Result(Space(D5, a), List(SpaceError))\n Results in a 5-dimensional `Space` on success, or `SpaceErrors` on failure.\n\n ## Examples\n\n \n &gt; type Axis { A B C D E }\n &gt; assert Ok(space) = d5(#(A, 5), #(B, 4), #(C, 3), #(D, 2), #(E, 1))\n &gt; elements(space)\n [#(A, 5), #(B, 4), #(C, 3), #(D, 2), #(E, 1)]\n \n\nSynonyms:\nargamak/space.d5\nargamak/space d5","url":"argamak/space.html#d5"},{"doc":"argamak/space","title":"d6","content":"pub fn d6(a: #(a, Int), b: #(a, Int), c: #(a, Int), d: #(a, Int), e: #(\n    a,\n    Int,\n  ), f: #(a, Int)) -&gt; Result(Space(D6, a), List(SpaceError))\n Results in a 6-dimensional `Space` on success, or `SpaceErrors` on failure.\n\n ## Examples\n\n \n &gt; type Axis { A B C D E F }\n &gt; assert Ok(space) =\n &gt;   d6(#(A, 9), #(B, 9), #(C, 9), #(D, 9), #(E, 9), #(F, 9))\n &gt; elements(space)\n [#(A, 9), #(B, 9), #(C, 9), #(D, 9), #(E, 9), #(F, 9)]\n \n\nSynonyms:\nargamak/space.d6\nargamak/space d6","url":"argamak/space.html#d6"},{"doc":"argamak/space","title":"degree","content":"pub fn degree(of space: Space(a, b)) -&gt; a\n Returns the degree of a given `Space`.\n\n ## Examples\n\n \n &gt; assert Ok(space) = d0()\n &gt; degree(space)\n D0\n\n &gt; type Axis { A B C }\n &gt; assert Ok(space) = d1(#(A, -1))\n &gt; degree(space)\n D1\n\n &gt; assert Ok(space) = d3(#(A, 2), #(B, 2), #(C, -1))\n &gt; degree(space)\n D3\n \n\nSynonyms:\nargamak/space.degree\nargamak/space degree","url":"argamak/space.html#degree"},{"doc":"argamak/space","title":"elements","content":"pub fn elements(of space: Space(a, b)) -&gt; List(#(b, Int))\n Returns the elements of a given `Space`.\n\n ## Examples\n\n \n &gt; assert Ok(space) = d0()\n &gt; elements(space)\n []\n\n &gt; type Axis { A B C }\n &gt; assert Ok(space) = d1(#(A, -1))\n &gt; elements(space)\n [#(A, -1)]\n\n &gt; assert Ok(space) = d3(#(A, 2), #(B, 2), #(C, -1))\n &gt; elements(space)\n [#(A, 2), #(B, 2), #(C, -1)]\n \n\nSynonyms:\nargamak/space.elements\nargamak/space elements","url":"argamak/space.html#elements"},{"doc":"argamak/space","title":"map_elements","content":"pub fn map_elements(of space: Space(a, b), with fun: fn(#(b, Int)) -&gt;\n    #(c, Int)) -&gt; Result(Space(a, c), List(SpaceError))\n Results in a new `Space` with the same number of dimensions as the given\n `Space` on success, or `SpaceErrors` on failure.\n\n Applies the given function to each element of the `Space`.\n\n ## Examples\n\n \n &gt; type Axis { A B C }\n &gt; assert Ok(space) = d0()\n &gt; assert Ok(space) = map_elements(of: space, with: fn(_) { #(C, 3) })\n &gt; elements(space)\n []\n\n &gt; assert Ok(space) = d1(#(A, -1))\n &gt; assert Ok(space) = map_elements(of: space, with: fn(_) { #(C, 3) })\n &gt; elements(space)\n [#(C, 3)]\n\n &gt; assert Ok(space) = d3(#(A, -1), #(B, 2), #(C, 2))\n &gt; assert Ok(space) = map_elements(of: space, with: fn(element) {\n &gt;   let #(axis, size) = element\n &gt;   case size == -1 {\n &gt;     True -&gt; #(axis, 4)\n &gt;     False -&gt; element\n &gt;   }\n &gt; })\n &gt; elements(space)\n [#(A, 4), #(B, 2), #(C, 2)]\n \n\nSynonyms:\nargamak/space.map_elements\nargamak/space map_elements","url":"argamak/space.html#map_elements"},{"doc":"argamak/space","title":"shape","content":"pub fn shape(of space: Space(a, b)) -&gt; List(Int)\n Returns the shape of a given `Space`.\n\n ## Examples\n\n \n &gt; assert Ok(space) = d0()\n &gt; shape(space)\n []\n\n &gt; type Axis { A B C }\n &gt; assert Ok(space) = d1(#(A, -1))\n &gt; shape(space)\n [-1]\n\n &gt; assert Ok(space) = d3(#(A, 2), #(B, 2), #(C, -1))\n &gt; shape(space)\n [2, 2, -1]\n \n\nSynonyms:\nargamak/space.shape\nargamak/space shape","url":"argamak/space.html#shape"},{"doc":"argamak/space","title":"to_string","content":"pub fn to_string(space: Space(a, b)) -&gt; String\n Converts a `Space` into a `String`.\n\n ## Examples\n\n \n &gt; assert Ok(space) = d0()\n &gt; to_string(space)\n &quot;D0&quot;\n\n &gt; type Axis { A B C }\n &gt; assert Ok(space) = d1(#(A, -1))\n &gt; to_string(space)\n &quot;D1 #(A, -1)&quot;\n\n &gt; assert Ok(space) = d3(#(A, 2), #(B, 2), #(C, -1))\n &gt; to_string(space)\n &quot;D3 #(A, 2), #(B, 2), #(C, -1)&quot;\n \n\nSynonyms:\nargamak/space.to_string\nargamak/space to_string","url":"argamak/space.html#to_string"},{"doc":"argamak/space","title":"argamak/space","content":"","url":"argamak/space.html"},{"doc":"argamak/tensor","title":"Native","content":"pub external type Native\n A type for `Native` tensor representations.\n\n\nSynonyms:\nargamak/tensor.Native\nargamak/tensor Native","url":"argamak/tensor.html#Native"},{"doc":"argamak/tensor","title":"Tensor","content":"pub opaque type Tensor(a, dn, axis)\n A `Tensor` is a generic container for n-dimensional data structures.\n\n\nSynonyms:\nargamak/tensor.Tensor\nargamak/tensor Tensor","url":"argamak/tensor.html#Tensor"},{"doc":"argamak/tensor","title":"TensorError","content":"pub type TensorError {\n  EmptyTensor\n  IncompatibleAxes\n  IncompatibleShape\n  InvalidData\n  SpaceErrors(space.SpaceErrors)\n}\n When a tensor operation cannot succeed.\n\nEmptyTensor\n\n\nIncompatibleAxes\n\n\nIncompatibleShape\n\n\nInvalidData\n\n\nSpaceErrors(space.SpaceErrors)\n\n\nSynonyms:\nargamak/tensor.TensorError\nargamak/tensor TensorError","url":"argamak/tensor.html#TensorError"},{"doc":"argamak/tensor","title":"as_format","content":"pub fn as_format(for tensor: Tensor(a, b, c), apply format: fn() -&gt;\n    Format(d)) -&gt; Tensor(d, b, c)\n Changes the `Format` of a `Tensor`.\n\n Converting from float formats to integer formats truncates the data. For\n consistency, consider using `round`, `floor`, or `ceil` before casting from\n float formats to integer formats.\n\n Lowering precision may lead to an overflow or underflow, the outcome of\n which depends on platform and compiler.\n\n ## Examples\n\n \n &gt; import argamak/format\n &gt; as_format(for: from_int(0), apply: format.float32)\n from_float(0.)\n\n &gt; import argamak/space\n &gt; type Axis { X }\n &gt; assert Ok(space) = space.d1(#(X, -1))\n &gt; try tensor = from_floats(of: [1., 2., 3.], into: space)\n &gt; Ok(as_format(for: tensor, apply: format.int32))\n from_ints(of: [1, 2, 3], into: space)\n \n\nSynonyms:\nargamak/tensor.as_format\nargamak/tensor as_format","url":"argamak/tensor.html#as_format"},{"doc":"argamak/tensor","title":"axes","content":"pub fn axes(tensor: Tensor(a, b, c)) -&gt; List(c)\n Returns the axes of a given `Tensor`.\n\n ## Examples\n\n \n &gt; axes(from_int(3))\n []\n\n &gt; import argamak/space\n &gt; type Axis { X }\n &gt; assert Ok(space) = space.d1(#(X, -1))\n &gt; try tensor = from_floats(of: [1., 2., 3.], into: space)\n &gt; Ok(axes(tensor))\n Ok([X])\n\n &gt; type OtherAxis { Alpha Omega }\n &gt; assert Ok(space) = space.d2(#(Alpha, 1), #(Omega, 3))\n &gt; try tensor = from_ints(of: [1, 2, 3], into: space)\n &gt; Ok(axes(tensor))\n Ok([Alpha, Omega])\n \n\nSynonyms:\nargamak/tensor.axes\nargamak/tensor axes","url":"argamak/tensor.html#axes"},{"doc":"argamak/tensor","title":"broadcast","content":"pub fn broadcast(from tensor: Tensor(a, b, c), into new_space: Space(\n    d,\n    e,\n  )) -&gt; Result(Tensor(a, d, e), TensorError)\n Results in a `Tensor` broadcast into a given n-dimensional `Space` on\n success, or a `TensorError` on failure.\n\n The new `Space` cannot be of lower dimensionality than the `Tensor`&#39;s\n current `Space`.\n\n The new `Tensor`&#39;s current `Space` must have dimension sizes compatible with\n the right-most dimensions of the new `Space`; that is, every current\n dimension size must be `1` or equal to its counterpart in the new `Space`.\n\n ## Examples\n\n \n &gt; import argamak/space\n &gt; type Axis { X Y Z }\n &gt; let tensor = from_int(0)\n &gt; assert Ok(space) = space.d1(#(X, 3))\n &gt; try tensor = broadcast(from: tensor, into: space)\n &gt; Ok(print(from: tensor, wrap_at: -1, meta: True))\n // Tensor\n // format: Int32\n // space: D1 #(X, 3)\n // data:\n // [0, 0, 0]\n Ok(Nil)\n\n &gt; assert Ok(space) = space.d1(#(X, -1))\n &gt; let tensor = from_ints(of: [-1], into: space)\n &gt; assert Ok(space) = space.d1(#(Y, 5))\n &gt; try tensor = broadcast(from: tensor, into: space)\n &gt; Ok(print(from: tensor, wrap_at: -1, meta: True))\n // Tensor\n // format: Int32\n // space: D1 #(Y, 5)\n // data:\n // [-1, -1, -1, -1, -1]\n Ok(Nil)\n\n &gt; assert Ok(space) = space.d1(#(X, -1))\n &gt; try tensor = from_floats(of: [1., 2., 3.], into: space)\n &gt; assert Ok(space) = space.d2(#(X, 2), #(Y, 3))\n &gt; try tensor = broadcast(from: tensor, into: space)\n &gt; Ok(print(from: tensor, wrap_at: -1, meta: True))\n // Tensor\n // format: Float32\n // space: D2 #(X, 2), #(Y, 3)\n // data:\n // [[1.0, 2.0, 3.0],\n //  [1.0, 2.0, 3.0]]\n Ok(Nil)\n \n\nSynonyms:\nargamak/tensor.broadcast\nargamak/tensor broadcast","url":"argamak/tensor.html#broadcast"},{"doc":"argamak/tensor","title":"broadcast_over","content":"pub fn broadcast_over(from tensor: Tensor(a, b, c), into new_space: Space(\n    d,\n    e,\n  ), with axes: fn(#(c, Int)) -&gt; e) -&gt; Result(\n  Tensor(a, d, e),\n  TensorError,\n)\n Results in a `Tensor` broadcast into a given n-dimensional `Space` on\n success, or a `TensorError` on failure.\n\n The new `Space` cannot be of lower dimensionality than the `Tensor`&#39;s\n current `Space`.\n\n The given function will be used to map from the elements of the `Tensor`&#39;s\n current `Space` to axes of the new `Space`, allowing broadcasting into a\n `Space` that would be incompatible for a standard `broadcast` operation.\n The function must uniquely map the current `Space`&#39;s axes to the new\n `Space`&#39;s axes; that is, duplicate axes will result in a `TensorError`.\n Furthermore, the current `Space`&#39;s axes must maintain the same relative\n order when mapped to the new `Space`.\n\n The new `Tensor`&#39;s current `Space` must have dimension sizes compatible with\n the matching dimensions of the new `Space`; that is, every current dimension\n size must be `1` or equal to its counterpart in the new `Space`.\n\n ## Examples\n\n \n &gt; import argamak/space\n &gt; type Axis { X Y Z }\n &gt; assert Ok(space) = space.d1(#(X, -1))\n &gt; try tensor = from_floats(of: [1., 2., 3.], into: space)\n &gt; assert Ok(space) = space.d2(#(X, 3), #(Y, 2))\n &gt; try new_tensor = broadcast_over(\n     from: tensor,\n     into: space,\n     with: fn(_) { X },\n   )\n &gt; Ok(print(from: new_tensor, wrap_at: -1, meta: True))\n // Tensor\n // format: Float32\n // space: D2 #(X, 3), #(Y, 2)\n // data:\n // [[1.0, 1.0],\n //  [2.0, 2.0],\n //  [3.0, 3.0]]\n Ok(Nil)\n\n &gt; try tensor = from_ints(of: [1, 2, 3, 4, 5, 6], into: space)\n &gt; assert Ok(space) = space.d3(#(X, 3), #(Y, 2), #(Z, 2))\n &gt; try new_tensor = broadcast_over(\n     from: tensor,\n     into: space,\n     with: fn(element) {\n       let #(axis, _size) = element\n       case axis {\n         X -&gt; X\n         Y -&gt; Z\n       }\n     },\n   )\n &gt; Ok(print(from: new_tensor, wrap_at: -1, meta: True))\n // Tensor\n // format: Int32\n // space: D3 #(X, 3), #(Y, 2), #(Z, 2)\n // data:\n // [[[1, 2],\n //   [1, 2]],\n //  [[3, 4],\n //   [3, 4]],\n //  [[5, 6],\n //   [5, 6]]]\n Ok(Nil)\n\n &gt; import gleam/pair\n &gt; try new_tensor = broadcast_over(\n     from: tensor,\n     into: space,\n     with: pair.first,\n   )\n &gt; Ok(print(from: new_tensor, wrap_at: -1, meta: True))\n // Tensor\n // format: Int32\n // space: D3 #(X, 3), #(Y, 2), #(Z, 2)\n // data:\n // [[[1, 1],\n //   [2, 2]],\n //  [[3, 3],\n //   [4, 4]],\n //  [[5, 5],\n //   [6, 6]]]\n Ok(Nil)\n \n\nSynonyms:\nargamak/tensor.broadcast_over\nargamak/tensor broadcast_over","url":"argamak/tensor.html#broadcast_over"},{"doc":"argamak/tensor","title":"format","content":"pub fn format(tensor: Tensor(a, b, c)) -&gt; Format(a)\n Returns the `Format` of a given `Tensor`.\n\n See `argamak/format` for more information.\n\n ## Examples\n\n \n import argamak/format\n &gt; format(from_float(0.))\n format.float32()\n\n &gt; import argamak/space\n &gt; type Axis { X }\n &gt; assert Ok(space) = space.d1(#(X, -1))\n &gt; try tensor = from_ints(of: [1, 2, 3], into: space)\n &gt; Ok(format(tensor))\n Ok(format.int32())\n \n\nSynonyms:\nargamak/tensor.format\nargamak/tensor format","url":"argamak/tensor.html#format"},{"doc":"argamak/tensor","title":"from_float","content":"pub fn from_float(float: Float) -&gt; Tensor(Float, D0, Nil)\n Converts a `Float` into a `Tensor`.\n\n ## Examples\n\n \n &gt; let tensor = from_float(1.)\n &gt; print(from: tensor, wrap_at: -1, meta: False)\n // 1.0\n Nil\n \n\nSynonyms:\nargamak/tensor.from_float\nargamak/tensor from_float","url":"argamak/tensor.html#from_float"},{"doc":"argamak/tensor","title":"from_floats","content":"pub fn from_floats(of list: List(Float), into space: Space(a, b)) -&gt; Result(\n  Tensor(Float, a, b),\n  TensorError,\n)\n Results in a `Tensor` created from a list of floats and placed into a given\n n-dimensional `Space` on success, or a `TensorError` on failure.\n\n The `Space`&#39;s `Shape` may have a single dimension given as `-1`, in which\n case that dimension&#39;s size will be inferred from the given list. This is\n useful when working with lists of unknown length.\n\n ## Examples\n\n \n &gt; import argamak/space\n &gt; type Axis { X Y Z }\n &gt; assert Ok(space) = space.d1(#(X, -1))\n &gt; try tensor = from_floats(of: [1.], into: space)\n &gt; Ok(print(from: tensor, wrap_at: -1, meta: True))\n // Tensor\n // format: Float32\n // space: D1 #(X, 1)\n // data:\n // [1.0]\n Ok(Nil)\n\n &gt; assert Ok(space) = space.d2(#(X, 2), #(Y, 2))\n &gt; try tensor = from_floats(of: [1., 2., 3., 4.], into: space)\n &gt; Ok(print(from: tensor, wrap_at: -1, meta: True))\n // Tensor\n // format: Float32\n // space: D2 #(X, 2), #(Y, 2)\n // data:\n // [[1.0, 2.0],\n //  [3.0, 4.0]]\n Ok(Nil)\n\n &gt; assert Ok(space) = space.d3(#(X, -1), #(Y, 2), #(Z, 2))\n &gt; let list = [1., 2., 3., 4., 5., 6., 7., 8.]\n &gt; try tensor = from_floats(of: list, into: space)\n &gt; Ok(print(from: tensor, wrap_at: -1, meta: True))\n // Tensor\n // format: Float32\n // space: D3 #(X, 2), #(Y, 2), #(Z, 2)\n // data:\n // [[[1.0, 2.0],\n //   [3.0, 4.0]],\n //  [[5.0, 6.0],\n //   [7.0, 8.0]]]\n Ok(Nil)\n \n\nSynonyms:\nargamak/tensor.from_floats\nargamak/tensor from_floats","url":"argamak/tensor.html#from_floats"},{"doc":"argamak/tensor","title":"from_int","content":"pub fn from_int(int: Int) -&gt; Tensor(Int, D0, Nil)\n Converts an `Int` into a `Tensor`.\n\n ## Examples\n\n \n &gt; let tensor = from_int(1)\n &gt; print(from: tensor, wrap_at: -1, meta: False)\n // 1\n Nil\n \n\nSynonyms:\nargamak/tensor.from_int\nargamak/tensor from_int","url":"argamak/tensor.html#from_int"},{"doc":"argamak/tensor","title":"from_ints","content":"pub fn from_ints(of list: List(Int), into space: Space(a, b)) -&gt; Result(\n  Tensor(Int, a, b),\n  TensorError,\n)\n Results in a `Tensor` created from a list of integers and placed into a\n given n-dimensional `Space` on success, or a `TensorError` on failure.\n\n The `Space`&#39;s `Shape` may have a single dimension given as `-1`, in which\n case that dimension&#39;s size will be inferred from the given list. This is\n useful when working with lists of unknown length.\n\n ## Examples\n\n \n &gt; import argamak/space\n &gt; type Axis { X Y Z }\n &gt; assert Ok(space) = space.d1(#(X, -1))\n &gt; try tensor = from_ints(of: [1], into: space)\n &gt; Ok(print(from: tensor, wrap_at: -1, meta: True))\n // Tensor\n // format: Int32\n // space: D1 #(X, 1)\n // data:\n // [1]\n Ok(Nil)\n\n &gt; assert Ok(space) = space.d2(#(X, 2), #(Y, 2))\n &gt; try tensor = from_ints(of: [1, 2, 3, 4], into: space)\n &gt; Ok(print(from: tensor, wrap_at: -1, meta: True))\n // Tensor\n // format: Int32\n // space: D2 #(X, 2), #(Y, 2)\n // data:\n // [[1, 2],\n //  [3, 4]]\n Ok(Nil)\n\n &gt; assert Ok(space) = space.d3(#(X, -1), #(Y, 2), #(Z, 2))\n &gt; let list = [1, 2, 3, 4, 5, 6, 7, 8]\n &gt; try tensor = from_ints(of: list, into: space)\n &gt; Ok(print(from: tensor, wrap_at: -1, meta: True))\n // Tensor\n // format: Int32\n // space: D3 #(X, 2), #(Y, 2), #(Z, 2)\n // data:\n // [[[1, 2],\n //   [3, 4]],\n //  [[5, 6],\n //   [7, 8]]]\n Ok(Nil)\n \n\nSynonyms:\nargamak/tensor.from_ints\nargamak/tensor from_ints","url":"argamak/tensor.html#from_ints"},{"doc":"argamak/tensor","title":"from_native","content":"pub fn from_native(of native: Native, into space: Space(a, b), with format: fn() -&gt;\n    Format(c)) -&gt; Result(Tensor(c, a, b), TensorError)\n Results in a `Tensor` created from a `Native` representation on success, or\n a `TensorError` on failure.\n\n ## Examples\n\n \n &gt; import argamak/format\n &gt; import argamak/space\n &gt; import gleam/dynamic.{Dynamic}\n &gt; external fn erlang_tensor(Dynamic) -&gt; Native =\n &gt;   &quot;Elixir.Nx&quot; &quot;tensor&quot;\n &gt; let native = erlang_tensor(dynamic.from([[1, 2], [3, 4]]))\n &gt; assert Ok(space) = space.d2(#(X, 2), #(Y, -1))\n &gt; try tensor = from_native(of: native, into: space, with: format.int32)\n &gt; Ok(print(from: tensor, wrap_at: -1, meta: True))\n // Tensor\n // format: Int32\n // space: D2 #(X, 2), #(Y, 2)\n // data:\n // [[1, 2],\n //  [3, 4]]\n Ok(Nil)\n \n\nSynonyms:\nargamak/tensor.from_native\nargamak/tensor from_native","url":"argamak/tensor.html#from_native"},{"doc":"argamak/tensor","title":"print","content":"pub fn print(from tensor: Tensor(a, b, c), wrap_at max_width: Int, meta meta: Bool) -&gt; Nil\n Prints a `Tensor`&#39;s underlying data to standard out.\n\n Takes a `max_width` (in columns) argument for which the special values `-1`\n and `0` represent default and no wrapping, respectively.\n\n If the `meta` argument is `True`, various `Tensor` details will be printed\n with the data.\n\n ## Examples\n\n \n &gt; import argamak/space\n &gt; type Axis { X Y Z }\n &gt; assert Ok(space) = space.d2(#(X, 2), #(Y, 2))\n &gt; try tensor = from_ints(of: [1, 2, 3, 4], into: space)\n &gt; Ok(print(from: tensor, wrap_at: -1, meta: False))\n // [[1, 2],\n //  [3, 4]]\n Ok(Nil)\n\n &gt; assert Ok(space) = space.d3(#(X, 2), #(Y, 1), #(Z, 4))\n &gt; let list = [1, 2, 3, 4, 5, 6, 7, 8]\n &gt; try tensor = from_ints(of: list, into: space)\n &gt; Ok(print(from: tensor, wrap_at: 10, meta: True))\n // Tensor\n // format: Int64\n // space: D3 #(X, 2), #(Y, 1), #(Z, 4)\n // data:\n // [[[1, 2,\n //    3, 4]],\n //  [[5, 6,\n //    7, 8]]]\n Ok(Nil)\n\nSynonyms:\nargamak/tensor.print\nargamak/tensor print","url":"argamak/tensor.html#print"},{"doc":"argamak/tensor","title":"rank","content":"pub fn rank(tensor: Tensor(a, b, c)) -&gt; Int\n Returns the rank of a given `Tensor` as an `Int` representing the number of\n dimensions.\n\n ## Examples\n\n \n &gt; rank(from_float(0.))\n 0\n\n &gt; import argamak/space\n &gt; type Axis { X Y Z }\n &gt; assert Ok(space) = space.d1(#(X, -1))\n &gt; try tensor = from_ints(of: [1, 2, 3], into: space)\n &gt; Ok(rank(tensor))\n Ok(1)\n\n &gt; assert Ok(space) = space.d3(#(X, 2), #(Y, 2), #(Z, 2))\n &gt; try tensor = from_ints(of: [1, 2, 3, 4, 5, 6, 7, 8], into: space)\n &gt; Ok(rank(tensor))\n Ok(3)\n \n\nSynonyms:\nargamak/tensor.rank\nargamak/tensor rank","url":"argamak/tensor.html#rank"},{"doc":"argamak/tensor","title":"reshape","content":"pub fn reshape(put tensor: Tensor(a, b, c), into new_space: Space(\n    d,\n    e,\n  )) -&gt; Result(Tensor(a, d, e), TensorError)\n Results in a `Tensor` placed into a given n-dimensional `Space` on success,\n or a `TensorError` on failure.\n\n The `Space`&#39;s `Shape` may have a single dimension given as `-1`, in which\n case that dimension&#39;s size will be inferred from the given list. This is\n useful when working with lists of unknown length.\n\n ## Examples\n\n \n &gt; import argamak/space\n &gt; type Axis { X Y Z }\n &gt; let tensor = from_float(1.)\n &gt; assert Ok(space) = space.d1(#(X, -1))\n &gt; try tensor = reshape(put: tensor, into: space)\n &gt; Ok(print(from: tensor, wrap_at: -1, meta: True))\n // Tensor\n // format: Float32\n // space: D1 #(X, 1)\n // data:\n // [1.0]\n Ok(Nil)\n\n &gt; assert Ok(space) = space.d1(#(X, -1))\n &gt; try tensor = from_floats(of: [1., 2., 3., 4.], into: space)\n &gt; assert Ok(space) = space.d2(#(X, 2), #(Y, 2))\n &gt; try tensor = reshape(put: tensor, into: space)\n &gt; Ok(print(from: tensor, wrap_at: -1, meta: True))\n // Tensor\n // format: Float32\n // space: D2 #(X, 2), #(Y, 2)\n // data:\n // [[1.0, 2.0],\n //  [3.0, 4.0]]\n Ok(Nil)\n\n &gt; assert Ok(space) = space.d2(#(X, 2), #(Y, -1))\n &gt; let list = [1., 2., 3., 4., 5., 6., 7., 8.]\n &gt; try tensor = from_floats(of: list, into: space)\n &gt; assert Ok(space) = space.d3(#(X, -1), #(Y, 2), #(Z, 2))\n &gt; try tensor = reshape(put: tensor, into: space)\n &gt; Ok(print(from: tensor, wrap_at: -1, meta: True))\n // Tensor\n // format: Float32\n // space: D3 #(X, 2), #(Y, 2), #(Z, 2)\n // data:\n // [[[1.0, 2.0],\n //   [3.0, 4.0]],\n //  [[5.0, 6.0],\n //   [7.0, 8.0]]]\n Ok(Nil)\n\n &gt; assert Ok(space) = space.d3(#(X, 1), #(Y, 1), #(Z, 1))\n &gt; try tensor = from_floats(of: [1.], into: space)\n &gt; assert Ok(space) = space.d0()\n &gt; try tensor = reshape(put: tensor, into: space)\n &gt; Ok(print(from: tensor, wrap_at: -1, meta: False))\n // 1.0\n Ok(Nil)\n \n\nSynonyms:\nargamak/tensor.reshape\nargamak/tensor reshape","url":"argamak/tensor.html#reshape"},{"doc":"argamak/tensor","title":"shape","content":"pub fn shape(tensor: Tensor(a, b, c)) -&gt; List(Int)\n Returns the shape of a given `Tensor`.\n\n ## Examples\n\n \n &gt; shape(from_float(0.))\n []\n\n &gt; import argamak/space\n &gt; type Axis { X Y Z }\n &gt; assert Ok(space) = space.d1(#(X, -1))\n &gt; try tensor = from_ints(of: [1, 2, 3], into: space)\n &gt; Ok(shape(tensor))\n Ok([3])\n\n &gt; assert Ok(space) = space.d3(#(X, 2), #(Y, 2), #(Z, 2))\n &gt; try tensor = from_ints(of: [1, 2, 3, 4, 5, 6, 7, 8], into: space)\n &gt; Ok(shape(tensor))\n Ok([2, 2, 2])\n \n\nSynonyms:\nargamak/tensor.shape\nargamak/tensor shape","url":"argamak/tensor.html#shape"},{"doc":"argamak/tensor","title":"space","content":"pub fn space(tensor: Tensor(a, b, c)) -&gt; Space(b, c)\n Returns the `Space` a given `Tensor` is currently in.\n\n ## Examples\n\n \n &gt; import argamak/space\n &gt; assert Ok(space) = space.d0()\n &gt; space(from_float(0.))\n space\n\n &gt; type Axis { X Y Z }\n &gt; assert Ok(space) = space.d1(#(X, -1))\n &gt; try tensor = from_ints(of: [1, 2, 3], into: space)\n &gt; Ok(space(tensor))\n Ok(space)\n\n &gt; assert Ok(space) = space.d3(#(X, 2), #(Y, 2), #(Z, 2))\n &gt; try tensor = from_ints(of: [1, 2, 3, 4, 5, 6, 7, 8], into: space)\n &gt; Ok(space(tensor))\n Ok(space)\n \n\nSynonyms:\nargamak/tensor.space\nargamak/tensor space","url":"argamak/tensor.html#space"},{"doc":"argamak/tensor","title":"to_float","content":"pub fn to_float(tensor: Tensor(Float, D0, a)) -&gt; Float\n Converts a `Tensor` without dimensions into a `Float`.\n\n ## Examples\n\n \n &gt; let tensor = from_float(0.)\n &gt; to_float(tensor)\n 0.\n\n &gt; import argamak/format\n &gt; let tensor = from_int(0)\n &gt; let tensor = as_format(for: tensor, apply: format.float32)\n &gt; to_float(tensor)\n 0.\n \n\nSynonyms:\nargamak/tensor.to_float\nargamak/tensor to_float","url":"argamak/tensor.html#to_float"},{"doc":"argamak/tensor","title":"to_int","content":"pub fn to_int(tensor: Tensor(Int, D0, a)) -&gt; Int\n Converts a `Tensor` without dimensions into an `Int`.\n\n ## Examples\n\n \n &gt; let tensor = from_int(0)\n &gt; to_int(tensor)\n 0\n\n &gt; import argamak/format\n &gt; let tensor = from_float(0.)\n &gt; let tensor = as_format(for: tensor, apply: format.int32)\n &gt; to_int(tensor)\n 0\n \n\nSynonyms:\nargamak/tensor.to_int\nargamak/tensor to_int","url":"argamak/tensor.html#to_int"},{"doc":"argamak/tensor","title":"to_list","content":"pub fn to_list(tensor: Tensor(a, b, c)) -&gt; List(a)\n Converts a `Tensor` into a flat list of numbers.\n\n ## Examples\n\n \n &gt; to_list(from_int(0))\n [0]\n\n &gt; import argamak/space\n &gt; type Axis { X Y }\n &gt; assert Ok(space) = space.d2(#(X, 3), #(Y, 1))\n &gt; try tensor = from_floats(of: [1., 2., 3.], into: space)\n &gt; Ok(to_list(tensor))\n Ok([1., 2., 3.])\n \n\nSynonyms:\nargamak/tensor.to_list\nargamak/tensor to_list","url":"argamak/tensor.html#to_list"},{"doc":"argamak/tensor","title":"to_native","content":"pub fn to_native(tensor: Tensor(a, b, c)) -&gt; Native\n Coverts a `Tensor` into its `Native` representation.\n\n ## Examples\n\n \n &gt; external fn erlang_rank(Native) -&gt; Int =\n &gt;   &quot;Elixir.Nx&quot; &quot;rank&quot;\n &gt; let native = to_native(from_int(3))\n &gt; erlang_rank(native)\n 0\n \n\nSynonyms:\nargamak/tensor.to_native\nargamak/tensor to_native","url":"argamak/tensor.html#to_native"},{"doc":"argamak/tensor","title":"argamak/tensor","content":"","url":"argamak/tensor.html"}]);